# langgraph.py - Специализированный сервер LangGraph

## 🎯 Назначение

Специализированный MCP сервер для работы с документацией LangGraph. Представляет собой упрощенную версию основного сервера, оптимизированную для быстрого доступа к документации LangGraph от LangChain AI.

## 📋 Основные характеристики

### Простота использования
- Один предопределенный домен: `https://langchain-ai.github.io/`
- Не требует конфигурации - работает "из коробки"
- Оптимизирован для тестирования MCP функциональности

### Безопасность
- Жесткий контроль домена - только LangGraph документация
- Нет поддержки локальных файлов
- Нет настройки allowed_domains

## 🛠️ Архитектура модуля

### Константы
```python
ALLOWED_PREFIX = "https://langchain-ai.github.io/"
```
**Назначение**: Единственный разрешенный префикс для всех запросов.

### HTTP клиент
```python
HTTPX_CLIENT = httpx.AsyncClient(follow_redirects=False)
```
**Особенности**:
- Глобальный клиент для переиспользования соединений
- Редиректы отключены для безопасности
- Timeout не настраивается (использует значение по умолчанию)

### Сервер
```python
server = FastMCP(name="llms-txt")
```
**Конфигурация**: Минимальная настройка без дополнительных инструкций.

## 🔧 MCP Инструмент

### get_docs(url: str = "overview") -> str

**Описание**: Единственный MCP инструмент для получения документации LangGraph.

**Параметры**:
- `url` (str): URL для загрузки или "overview" для получения главной страницы

**Логика работы**:
1. **Обработка "overview"**: Если url = "overview", заменяется на llms.txt URL
2. **Валидация домена**: Проверка, что URL начинается с разрешенного префикса
3. **HTTP запрос**: Синхронный запрос к указанному URL
4. **Конвертация**: HTML → Markdown через markdownify
5. **Возврат результата**: Markdown контент или сообщение об ошибке

**Пример использования**:
```python
# Получить индекс документации
result = await get_docs("overview")

# Получить конкретную страницу
result = await get_docs("https://langchain-ai.github.io/langgraph/concepts/")
```

## 🔄 Алгоритм работы

```mermaid
graph TD
    A[get_docs вызван] --> B{url == "overview"?}
    B -->|Да| C[url = llms.txt URL]
    B -->|Нет| D[Проверить префикс]
    C --> D
    D --> E{Префикс корректный?}
    E -->|Нет| F[Вернуть ошибку]
    E -->|Да| G[HTTP GET запрос]
    G --> H{Статус == 200?}
    H -->|Нет| I[Вернуть ошибку]
    H -->|Да| J[Конвертировать в Markdown]
    J --> K[Вернуть результат]
```

## 🎭 Особенности реализации

### 1. Специальная обработка "overview"
```python
if url == "overview":
    url = "https://langchain-ai.github.io/langgraph/llms.txt"
```
**Обоснование**: Предоставляет простой способ получить список всех доступных документов.

### 2. Строгая валидация URL
```python
if not url.startswith(ALLOWED_PREFIX):
    return (
        "Error: Invalid url. Must start with https://langchain-ai.github.io/ "
        'or be "overview"'
    )
```
**Обоснование**: Максимальная безопасность - доступ только к документации LangGraph.

### 3. Простая обработка ошибок
```python
if response.status_code == 200:
    markdown_content = markdownify(response.text)
    return markdown_content
else:
    return "Encountered an error while fetching the URL."
```
**Обоснование**: Минимальная обработка ошибок для простоты использования.

## 🔗 Зависимости

### Внешние библиотеки
- `httpx`: HTTP клиент для запросов
- `markdownify`: Конвертация HTML в Markdown  
- `mcp.server.fastmcp`: MCP сервер фреймворк

### Стандартные модули
Модуль не использует стандартные библиотеки Python, что делает его очень легким.

## 🎯 Сценарии использования

### 1. Тестирование MCP
```bash
python -m superdocs.langgraph
```
**Применение**: Быстрый способ протестировать MCP функциональность без настройки.

### 2. Разработка и отладка
- Простой сервер для проверки интеграции с IDE
- Тестирование конвертации HTML в Markdown
- Проверка работы MCP инструментов

### 3. Демонстрация возможностей
- Показ работы с документацией через MCP
- Пример минимальной реализации MCP сервера

## 📊 Сравнение с основным сервером

| Характеристика | main.py | langgraph.py |
|----------------|---------|---------------|
| Конфигурация | Гибкая (YAML/JSON/CLI) | Фиксированная |
| Источники | Множественные | Один (LangGraph) |
| Локальные файлы | Поддерживаются | Нет |
| Безопасность | Настраиваемая | Максимальная |
| Сложность | Высокая | Минимальная |
| MCP инструменты | 2 (list + fetch) | 1 (get_docs) |
| Обработка ошибок | Детальная | Базовая |

## 🚀 Расширения и модификации

### Текущие ограничения
1. **Один домен**: Только LangGraph документация
2. **Нет кэширования**: Каждый запрос загружает данные заново
3. **Простые ошибки**: Минимальная диагностика проблем
4. **Фиксированная конфигурация**: Нет настроек

### Потенциальные улучшения
1. **Поддержка других доменов LangChain**: LangChain, LangSmith
2. **Кэширование**: Добавление простого in-memory кэша
3. **Расширенные ошибки**: Более информативные сообщения об ошибках
4. **Конфигурация**: Возможность изменения базового домена

### Пример расширения
```python
# Потенциальное расширение для поддержки LangChain
ALLOWED_DOMAINS = [
    "https://langchain-ai.github.io/",
    "https://python.langchain.com/",
    "https://smith.langchain.com/"
]

@server.tool()
async def get_langchain_docs(source: str = "langgraph", url: str = "overview") -> str:
    """Получить документацию из различных источников LangChain"""
    # Логика выбора домена на основе source
```

## 🧪 Тестирование

### Тестовые сценарии
1. **Запрос overview**: Должен вернуть llms.txt содержимое
2. **Валидные URL**: Должны загружаться успешно
3. **Невалидные URL**: Должны возвращать ошибку
4. **Недоступные страницы**: Должны обрабатываться корректно

### Примеры тестов
```python
# Тест получения overview
result = await get_docs("overview")
assert "LangGraph" in result

# Тест валидного URL
result = await get_docs("https://langchain-ai.github.io/langgraph/concepts/")
assert result != "Error"

# Тест невалидного домена
result = await get_docs("https://evil.com/docs")
assert "Error: Invalid url" in result
```

## 🎨 Стиль кода

### Простота превыше всего
- Глобальные переменные для простоты
- Минимум функций и классов
- Прямолинейная логика без абстракций

### Читаемость
- Понятные имена переменных
- Комментарии для важных участков
- Простые условные конструкции

### Надежность
- Явная проверка статуса HTTP ответа
- Обработка основных ошибок
- Валидация входных данных

Этот модуль представляет собой отличный пример того, как можно создать специализированный MCP сервер для конкретного случая использования, жертвуя гибкостью ради простоты и безопасности. 