# link_extractor.py - Модуль извлечения ссылок

## 🎯 Назначение

Простой и универсальный модуль для извлечения ссылок из HTML и Markdown контента. Следует принципу KISS (Keep It Simple, Stupid) - извлекает ВСЕ ссылки без попыток классификации или детекции типов. Обеспечивает универсальность работы с любым контентом документации.

## 📋 Основные компоненты

### 1. Класс данных Link

```python
@dataclass
class Link:
    title: str
    url: str  
    description: str = ""
```

**Назначение**: Простое и понятное представление ссылки с минимальным набором полей.

**Поля**:
- `title`: Текст ссылки (анкор)
- `url`: Абсолютный URL ссылки
- `description`: Опциональное описание (пока не используется)

### 2. Основной класс LinkExtractor

```python
class LinkExtractor:
    """Простой экстрактор ссылок без сложной логики."""
```

**Назначение**: Единственный класс модуля, инкапсулирующий всю логику извлечения ссылок.

## 🛠️ Публичные методы

### extract_all_links(content: str, base_url: str) -> List[Link]

**Описание**: Основной метод для извлечения всех ссылок из HTML или Markdown контента.

**Параметры**:
- `content`: HTML или Markdown контент для обработки
- `base_url`: Базовый URL для построения абсолютных ссылок

**Возвращает**: Список уникальных ссылок типа `Link`

**Логика работы**:
1. **Поиск Markdown ссылок**: Использует regex `\[([^\]]+)\]\(([^)]+)\)` для паттерна `[title](url)`
2. **Поиск HTML ссылок**: Использует regex `<a[^>]+href=["\']([^"\']+)["\'][^>]*>([^<]+)</a>` для `<a href="url">title</a>`
3. **Фильтрация**: Исключает якорные ссылки (`#`) и JavaScript (`javascript:`)
4. **Нормализация URL**: Конвертирует относительные ссылки в абсолютные через `urljoin()`
5. **Дедупликация**: Удаляет дублирующиеся ссылки по URL

**Примеры распознаваемых форматов**:
```markdown
[Getting Started](./getting-started)
[API Reference](https://example.com/api)
```

```html
<a href="/docs/guide">User Guide</a>
<a href="https://example.com/api">API Docs</a>
```

## 🔧 Приватные методы

### _deduplicate_links(links: List[Link]) -> List[Link]

**Описание**: Внутренний метод для удаления дублирующихся ссылок.

**Логика**:
- Отслеживает уникальные URL через `set`
- Сохраняет первое вхождение каждого URL
- Возвращает список без дубликатов

## 🏗️ Архитектурные принципы

### 1. Принцип KISS
```python
# Простые regex паттерны без сложной логики
markdown_links = re.findall(r'\[([^\]]+)\]\(([^)]+)\)', content)
html_links = re.findall(r'<a[^>]+href=["\']([^"\']+)["\'][^>]*>([^<]+)</a>', content)
```

**Обоснование**: Простота обеспечивает надежность и понятность кода.

### 2. Универсальность
- Работает как с HTML, так и с Markdown
- Не делает предположений о типах или категориях ссылок
- Извлекает ВСЕ ссылки без исключений

### 3. Безопасность
```python
# Фильтрация потенциально опасных ссылок
if not url.startswith('#') and not url.startswith('javascript:'):
```

**Обоснование**: Исключает локальные якоря и JavaScript код.

### 4. Нормализация
```python
full_url = urljoin(base_url, url)
```

**Обоснование**: Обеспечивает работоспособность относительных ссылок.

## 📊 Обработка разных типов контента

### Markdown документы
- Распознает стандартный синтаксис `[text](url)`
- Игнорирует inline код и code blocks (благодаря простому regex)
- Корректно обрабатывает ссылки с пробелами в названиях

### HTML документы  
- Извлекает ссылки из `<a>` тегов
- Поддерживает как одинарные, так и двойные кавычки в href
- Работает с различными форматами HTML (strict/loose)

### Смешанный контент
- Обрабатывает документы содержащие и HTML и Markdown
- Комбинирует результаты из обоих форматов
- Дедуплицирует ссылки независимо от источника

## 🔗 Зависимости

### Стандартная библиотека Python
- `re`: Регулярные выражения для поиска паттернов ссылок
- `urllib.parse.urljoin`: Построение абсолютных URL из относительных  
- `urllib.parse.urlparse`: Парсинг URL (косвенно через urljoin)
- `dataclasses`: Декоратор для создания Link dataclass
- `typing.List`: Типизация для списков ссылок

### Внешние зависимости
**Отсутствуют** - модуль использует только стандартную библиотеку Python.

## ⚡ Производительность

### Оптимизации
- Использование compiled regex паттернов (автоматически кешируются Python)
- Однопроходный алгоритм обработки контента
- Эффективная дедупликация через set

### Сложность алгоритма
- **Время**: O(n + m) где n - размер контента, m - количество найденных ссылок
- **Память**: O(m) для хранения результирующих ссылок

### Ограничения производительности
- Regex обработка больших документов (>1MB) может быть медленной
- Нет поддержки streaming обработки
- Загружает весь контент в память

## 🎯 Случаи использования

### В контексте SuperDocs
1. **Навигация по документации**: `browse_page` использует для извлечения ссылок
2. **Построение карты сайта**: Рекурсивный обход документации
3. **Валидация ссылок**: Проверка доступности найденных URL

### Ограничения применимости
- Не подходит для сложной классификации ссылок
- Не распознает специфические типы ссылок (download, external, etc.)
- Не обрабатывает JavaScript-генерируемые ссылки

## 🔮 Возможные расширения

### 1. Классификация ссылок
```python
@dataclass  
class ClassifiedLink(Link):
    link_type: str  # 'internal', 'external', 'download', etc.
    is_valid: bool  # результат валидации
```

### 2. Метаданные ссылок
```python
@dataclass
class EnhancedLink(Link):
    anchor_text: str    # полный текст якоря
    context: str        # окружающий контекст
    position: int       # позиция в документе
```

### 3. Конфигурируемые фильтры
```python
class ConfigurableLinkExtractor:
    def __init__(self, exclude_patterns: List[str] = None):
        self.exclude_patterns = exclude_patterns or []
```

### 4. Асинхронная валидация
```python
async def validate_links(self, links: List[Link]) -> List[Link]:
    """Асинхронная проверка доступности ссылок"""
```

## 🧪 Тестирование

### Рекомендуемые тест-кейсы
1. **Markdown ссылки**: Различные форматы и edge cases
2. **HTML ссылки**: Разные атрибуты и форматирование  
3. **Смешанный контент**: HTML + Markdown в одном документе
4. **Относительные ссылки**: Корректность urljoin
5. **Дедупликация**: Удаление одинаковых URL
6. **Фильтрация**: Исключение якорей и JavaScript

### Граничные случаи
- Пустой контент
- Контент без ссылок  
- Поврежденная разметка
- Очень длинные URL
- Unicode символы в ссылках 